import React from 'react';
import { withFormik } from 'formik';
import { string, object, array } from 'yup';
import { TextMutil } from './FormInputs';
import debounce from "debounce-promise";
import AsyncSelect from 'react-select/async';
import * as help from './help';

const questionSchema = object().shape({
    orderUid: string().required('OrderId is required'),
    orderId: string().required('OrderId is required'),
    content: string().required('Content is required'),
    refId: string().notRequired(),
    ctx: object().notRequired(),
    langs: array().of(string().notRequired()).notRequired()
});

const QuestionModel = withFormik({
    validationSchema: questionSchema,
    mapPropsToValues: props => props.data,
    handleSubmit: async (values, { props, setSubmitting, setErrors }) => {
        await props.onSave(values);
        setSubmitting(false);
    },
    displayName: 'new-question',
    validateOnChange: true
});

const defaultSourceTextOptions = [
    { label: 'General', value: help.defaultUuid }
];


class QuestionForm extends React.Component {
    constructor(props) {
        super(props);
        this.handleSearchText = debounce(this.handleSearchText, 1000);
    }
    render() {
        const {
            values,
            sourceOptions,
            touched,
            errors,
            isValid,
            handleBlur,
            handleSubmit,
            isSubmitting,
            handleChange,
            onClose
        } = this.props;

        const commonProps = {
            values,
            touched,
            errors,
            handleBlur,
            handleSubmit,
            handleChange,
            isSubmitting
        };

        const selectOptions = (sourceOptions && sourceOptions.length > 0) ? defaultSourceTextOptions.concat(sourceOptions) : defaultSourceTextOptions;

        return (
            <form onSubmit={handleSubmit}>
                <div className="form-row">
                    <div className="col-md-12">
                        <div className={`form-group ${errors['refId'] ? 'has-danger' : ''}`}>
                            <label className="form-label" htmlFor="qanda-source-text-inut">
                                Source Language Text
                            </label>
                            <AsyncSelect
                                id="qanda-source-text-inut"
                                isDisabled={isSubmitting}
                                cacheOptions={false}
                                defaultOptions={selectOptions}
                                loadOptions={this.handleSearchText}
                                onChange={this.handleSelectChange}
                                placeholder='Enter a source language text or select "General"'
                            />
                            <div className={errors['refId'] ? 'error' : ''}>
                                {errors['refId'] && <div className="input-feedback">{errors['refId']}</div>}
                            </div>
                        </div>
                    </div>
                </div>
                <div className="form-row">
                    <TextMutil
                        className="col-md-12"
                        id="content"
                        name="content"
                        label="Question"
                        placeholder="Enter your question"
                        {...commonProps}
                    >
                        >
                    </TextMutil>
                </div>
                <div className="btn-container d-flex flex-row justify-content-end" role="group" aria-label="newQuestionCommand">
                    <button type="button" aria-label="cancel" className="btn btn-primary" disabled={isSubmitting} onClick={onClose}>Cancel</button>
                    <button type="submit" aria-label="submit" className="btn btn-primary ml-2" disabled={isSubmitting || !isValid}>Add</button>
                </div>
            </form>
        );
    }

    handleSelectChange = data => {
        const { setFieldTouched, setFieldValue} = this.props;
        setFieldValue('refId', data.value);
        setFieldTouched('refId',true);
        setFieldValue('ctx', data.ctx);
        setFieldTouched('ctx',true);
    }

    handleSearchText = (input, callback) => this.props.searchText(input, callback);
};

export default QuestionModel(QuestionForm);