import React from 'react';
import { array } from 'prop-types';
import { AutoSizer, List, CellMeasurer, CellMeasurerCache } from 'react-virtualized';
import isEmpty from 'lodash/isEmpty'
import { TextInput } from './Inputs';
import QuestionGroup from './QuestionGroup';
import QandaContext from './QandaContext';

function filter(listgroups, filterText) {
    let list = [];

    if (!isEmpty(filterText)) {
        listgroups.forEach(group => {
            const [ctx, _] = group;
            const sourceText = ctx ? ctx.sourceText: null;

            if ((sourceText && sourceText.toLowerCase().indexOf(filterText.toLowerCase()) !== -1) ||
                (!sourceText && 'general' === filterText.toLowerCase())) {
                list.push(group);
            }
        });
    } else {
        list = listgroups;
    }

    return list;
};

export default class QuestionList extends React.PureComponent {
    static propTypes = {
        listgroups: array.isRequired,
    }

    constructor(props, context) {
        super(props, context);

        this.state = {
            listgroups: props.listgroups,
            selectedThread: null,
            filterText: ''
        };

        this._cache = new CellMeasurerCache({
            fixedWidth: true,
            minHeight: 50
        });

        this._rowRenderer = this._rowRenderer.bind(this);
    }

    static getDerivedStateFromProps(props, state) {
        if(props.listgroups !== state.cacheListGroups){
            return {
                cacheListGroups: props.listgroups,
                listgroups: filter(props.listgroups, state.filterText)
            }
        }
        return null;
    }

    onThreadSelect = selectedThread => {
        this.setState({ selectedThread });
    }

    onFilterChange = text => {
        const { listgroups, filterText } = this.props;
        if(text!==filterText){
            this.setState({
                filterText: text,
                listgroups: filter(listgroups, text)
            });
        } 
    }

    render() {
        const { listgroups, filterText, selectedThread } = this.state;
        return (
            <div className="qanda-list d-flex flex-column flex-fill">
                <div className="qanda-list-top container border-bottom shadow-sm">
                    <QandaContext.Consumer>
                        {({ onNewQuestion }) => (
                            <button type="button" className="btn btn-block btn-primary" onClick={onNewQuestion}>
                                <i className="far fa-question-circle"></i> Add Question
                        </button>
                        )}
                    </QandaContext.Consumer>

                    <form>
                        <div className="form-group">
                            <label htmlFor="input-search-source-text">Filter</label>
                            <TextInput onChange={this.onFilterChange} useDebounce clearable id="input-search-source-text" placeholder='Enter source language text or General' />
                        </div>
                    </form>
                </div>
                <div className="bg-light flex-fill mb-1">
                    <AutoSizer>
                        {({ width, height }) => (
                            (listgroups.length > 0) &&
                            <List
                                listgroups={listgroups}
                                filterText={filterText}
                                deferredMeasurementCache={this._cache}
                                overscanRowCount={3}
                                rowCount={listgroups.length}
                                rowHeight={this._cache.rowHeight}
                                rowRenderer={this._rowRenderer}
                                width={width}
                                height={height}
                                selectedThread={selectedThread}
                            />
                        )}
                    </AutoSizer>
                </div>
            </div>
        );
    }


    _rowRenderer({ index, key, parent, style }) {
        const { listgroups, selectedThread, filterText } = this.state;
        const rowdata = listgroups[(index % listgroups.length)];

        return (
            <CellMeasurer
                cache={this._cache}
                columnIndex={0}
                key={key}
                rowIndex={index}
                parent={parent}
            >
                {({ measure }) => (
                    <QuestionGroup
                        key={rowdata.id}
                        selectedThread={selectedThread}
                        data={rowdata}
                        style={style}
                        markText={filterText}
                        onThreadSelect={this.onThreadSelect}
                        measure={measure}
                    />
                )}
            </CellMeasurer>
        );
    }
}