import React from 'react';
import { bool, string, func, any } from 'prop-types';
import { TextInput } from './Inputs';
import { ConsumQandaContext } from './QandaContext';

const Comment = ({ author = 'You', content }) => {
    return (
        <div className="comment d-flex flex-row mt-3">
            <div className="avatar-icon">
                <i className="far fa-user-circle" />
            </div>

            <div className="thread-comment">
                <pre><span className="author">{author}</span>{content}</pre>
            </div>
        </div>
    );
}

const CommentInput = ({ author = 'You', submitting, onSubmit }) => {
    return (
        <div className="comment comment-input d-flex flex-row mt-4">
            <div className="avatar-icon">
                <i className="far fa-user-circle" />
            </div>
            <div className="thread-comment d-flex flex-row w-100 pr-2">
                <span className="author">{author}</span>
                <TextInput onChange={onSubmit} useEnter autofocus rows="2" className="w-100" loading={submitting} />
            </div>
        </div>
    );
};

const CommentList = ({ answers, members, authorName }) => Array.isArray(answers) ? answers.map(answer => (<Comment key={answer.id} author={authorName(members, answer.createdBy)} {...answer} />)) : null;

const Commands = ({ onReplyClick, onResolveClick, isResolved }) => {
    return (
        <div className="reaction-commands position-absolute btn-group btn-group-sm" role="group" aria-label="Basic example">
            <button type="button" className="btn btn-reply btn-outline-secondary" onClick={onReplyClick} disabled={isResolved}>
                Reply
            </button>
            <button type="button" className="btn btn-resolved btn-outline-secondary" onClick={onResolveClick}>
                {isResolved ? 'Re-open' : 'Resolve'}
            </button>
        </div>
    )
}

class QuestionThread extends React.PureComponent {
    static propTypes = {
        id: string,
        active: bool,
        thread: any,
        onSelect: func,
        measure: func
    }

    state = {
        submitting: false
    }

    render() {
        const { thread, active, authorName } = this.props;
        const { submitting } = this.state;
        const isResolved = thread.state === 8;
        return (
            <div className="question card position-relative">
                <Commands onReplyClick={this.handleOnReplyClick} onResolveClick={this.handleOnResolveClick} isResolved={isResolved}/>
                <div className={`card-body ${isResolved ? 'question-resolved':''}`}>
                    <div className="thread-start comment d-flex flex-row">
                        <Comment
                            author={authorName(thread.members, thread.createdBy)}
                            content={thread.content}
                        />
                    </div>
                    <div className="thread-follows container my-2 ml-2">
                        <CommentList answers={thread.answers} authorName={authorName} members={thread.members}></CommentList>
                        {active && !isResolved && (
                            <CommentInput onSubmit={this.handleSubmitComment} submitting={submitting} />
                        )}
                    </div>
                </div>
            </div>
        );
    }

    componentDidUpdate(prevProps, prevState) {
        if (prevProps.active !== this.props.active || prevProps.thread !== this.props.thread) {
            this.props.measure && this.props.measure();
        }
    }

    handleOnReplyClick = () => {
        const { id, onSelect } = this.props;
        if (onSelect) {
            onSelect(id);
        }
    };

    handleSubmitComment = content => {
        const { id, onSaveAnswer } = this.props;
        if (onSaveAnswer && (!!content && content.trim().length > 0)) {
            this.setState({ submitting: true });
            onSaveAnswer({ id, content })
                .then(_ => {
                    this.setState({ submitting: false });
                    this.props.measure && this.props.measure();
                })
                .catch(err => {
                    this.setState({ submitting: false });
                    Error.captureStackTrace(err);
                    throw err;
                });
        }
    };

    handleOnResolveClick = () => {
        const { thread, onResolveQuestion } = this.props;
        if (onResolveQuestion) {
            this.setState({ submitting: true });
            onResolveQuestion(thread)
                .then(_ => {
                    this.setState({ submitting: false });
                })
                .catch(err => {
                    this.setState({ submitting: false });
                    Error.captureStackTrace(err);
                    throw err;
                });
        }
    }
}

export default ConsumQandaContext(QuestionThread);